#pragma once
#include <luna/Result.h>

template <typename T> constexpr bool add_will_overflow(T a, T b)
{
    return __builtin_add_overflow_p(a, b, (T)0);
}

template <typename T> constexpr bool sub_will_overflow(T a, T b)
{
    return __builtin_sub_overflow_p(a, b, (T)0);
}

template <typename T> constexpr bool mul_will_overflow(T a, T b)
{
    return __builtin_mul_overflow_p(a, b, (T)0);
}

template <typename T> Result<T> safe_add(T a, T b)
{
    T result;

    if (__builtin_add_overflow(a, b, &result)) return err(EOVERFLOW);

    return result;
}

template <typename T> Result<T> safe_sub(T a, T b)
{
    T result;

    if (__builtin_sub_overflow(a, b, &result)) return err(EOVERFLOW);

    return result;
}

template <typename T> Result<T> safe_mul(T a, T b)
{
    T result;

    if (__builtin_mul_overflow(a, b, &result)) return err(EOVERFLOW);

    return result;
}